"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProxyProviderManager = void 0;
const core_1 = require("@nestjs/core");
const unknown_dependencies_exception_1 = require("@nestjs/core/errors/exceptions/unknown-dependencies.exception");
const cls_service_globals_1 = require("../cls-service.globals");
const proxy_provider_constants_1 = require("./proxy-provider.constants");
const proxy_provider_exceptions_1 = require("./proxy-provider.exceptions");
const proxy_provider_functions_1 = require("./proxy-provider.functions");
class ProxyProviderManager {
    static createProxyProvider(options) {
        var _a, _b;
        const providerSymbol = this.getProxyProviderSymbol(options);
        const proxy = this.createProxy(providerSymbol, (_a = options.type) !== null && _a !== void 0 ? _a : 'object');
        const proxyProvider = {
            provide: this.getProxyProviderToken(options),
            inject: [
                core_1.ModuleRef,
                ...((_b = options.inject) !== null && _b !== void 0 ? _b : []),
            ],
            useFactory: (moduleRef, ...injected) => {
                let providerOptions;
                if ((0, proxy_provider_functions_1.isProxyClassProviderOptions)(options)) {
                    this.throwIfClassHasNoProxyMetadata(options.useClass);
                    providerOptions = {
                        moduleRef,
                        token: options.provide,
                        useClass: options.useClass,
                    };
                }
                else {
                    providerOptions = {
                        injected,
                        token: options.provide,
                        useFactory: options.useFactory,
                    };
                }
                this.proxyProviderMap.set(providerSymbol, providerOptions);
                return proxy;
            },
        };
        return proxyProvider;
    }
    static createProxyProviderFromExistingKey(providerKey) {
        const proxy = this.createProxy(providerKey);
        const proxyProvider = {
            provide: providerKey,
            useValue: proxy,
        };
        return proxyProvider;
    }
    static getProxyProviderSymbol(options) {
        var _a, _b;
        const maybeExistingSymbol = typeof options.provide == 'symbol' ? options.provide : undefined;
        return (maybeExistingSymbol !== null && maybeExistingSymbol !== void 0 ? maybeExistingSymbol : Symbol.for((_b = (_a = options.provide) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : options.useClass
            .name));
    }
    static getProxyProviderToken(options) {
        var _a;
        return ((_a = options.provide) !== null && _a !== void 0 ? _a : options.useClass);
    }
    static createProxy(providerKey, type = 'object') {
        const getProvider = () => { var _a, _b; return (_b = (_a = this.clsService.get()) === null || _a === void 0 ? void 0 : _a[providerKey]) !== null && _b !== void 0 ? _b : {}; };
        const baseType = type === 'function' ? () => null : {};
        return new Proxy(baseType, {
            apply(_, thisArg, argArray) {
                return getProvider().apply(thisArg, argArray);
            },
            get(_, prop) {
                return getProvider()[prop];
            },
            set(_, prop, value) {
                return Reflect.set(getProvider(), prop, value);
            },
            ownKeys() {
                return Reflect.ownKeys(getProvider());
            },
            getPrototypeOf() {
                return Reflect.getPrototypeOf(getProvider());
            },
            getOwnPropertyDescriptor(_, prop) {
                return Reflect.getOwnPropertyDescriptor(getProvider(), prop);
            },
            has(_, prop) {
                return Reflect.has(getProvider(), prop);
            },
        });
    }
    static async resolveProxyProviders() {
        const promises = [...this.proxyProviderMap.keys()].map((providerSymbol) => this.resolveProxyProvider(providerSymbol));
        await Promise.all(promises);
    }
    static async resolveProxyProvider(providerSymbol) {
        if (this.clsService.get(providerSymbol)) {
            // skip resolution if the provider already exists in the CLS
            return;
        }
        const provider = this.proxyProviderMap.get(providerSymbol);
        if ((0, proxy_provider_functions_1.isProxyClassProvider)(provider)) {
            await this.resolveProxyClassProvider(providerSymbol, provider);
        }
        else {
            await this.resolveProxyFactoryProvider(providerSymbol, provider);
        }
    }
    static async resolveProxyClassProvider(providerSymbol, provider) {
        const moduleRef = provider.moduleRef;
        const Provider = provider.useClass;
        try {
            const proxyProvider = await moduleRef.create(Provider);
            this.clsService.set(providerSymbol, proxyProvider);
        }
        catch (error) {
            if (!(error instanceof unknown_dependencies_exception_1.UnknownDependenciesException))
                throw error;
            throw proxy_provider_exceptions_1.UnknownProxyDependenciesException.create(error, Provider);
        }
    }
    static throwIfClassHasNoProxyMetadata(Provider) {
        const hasMetadata = Reflect.getMetadata(proxy_provider_constants_1.CLS_PROXY_METADATA_KEY, Provider);
        if (!hasMetadata) {
            throw proxy_provider_exceptions_1.ProxyProviderNotDecoratedException.create(Provider);
        }
    }
    static async resolveProxyFactoryProvider(providerSymbol, provider) {
        const injected = provider.injected;
        const proxyProvider = await provider.useFactory.apply(null, injected);
        this.clsService.set(providerSymbol, proxyProvider);
    }
}
exports.ProxyProviderManager = ProxyProviderManager;
ProxyProviderManager.clsService = cls_service_globals_1.globalClsService;
ProxyProviderManager.proxyProviderMap = new Map();
//# sourceMappingURL=proxy-provider-manager.js.map